#! /usr/bin/env python

# import Nexus functions
from nexus import settings,job,get_machine,run_project
from nexus import generate_physical_system
from nexus import generate_pwscf,generate_pw2qmcpack
from nexus import generate_qmcpack,loop,linear,vmc,dmc

# Nexus settings
settings(
    pseudo_dir    = 'PSP/',
    runs          = '',
    results       = '',
    status_only   = 0,
    generate_only = 0,
    sleep         = 3,
    machine       = 'daint',
    account       = 's700',
    ) 

# specify job details
appdir = '/scratch/snx3000/amsmax/QMCPack/bin/'
dftjob  = job(sub_options='-C gpu',nodes=32,threads= 1,queue='normal',hours=1,app=appdir+'pw.x')
p2qjob  = job(sub_options='-C gpu',cores= 1,threads= 1,queue='normal',hours=1,app=appdir+'pw2qmcpack.x')
qmcjob  = job(sub_options='-C gpu',nodes=32,threads=12,queue='normal',hours=14,app=appdir+'qmcpack.cpu')
daint = get_machine('daint') 
daint.queue_size = 100

sims = []
# create 8 atom diamond structure
# specify k-point grids
kgrids = [(2,2,2),(4,4,4)]
directory = 'silicon'
first = True
for kgrid in kgrids:
    ks = '{0}{1}{2}'.format(*kgrid)
    
    # create conventional cell tiled from primitive one
    diamond = generate_physical_system(
        type      = 'crystal',
        structure = 'diamond',
        cell      = 'fcc',
        atoms      = 'Si',
        constants  = 5.430530000000000,
        units      = 'A',
        net_charge = 0,
        net_spin   = 0,
        Si         = 4,
        tiling     = [[2, -2, 2],[2, 2, -2],[-2, 2, 2]],
        kgrid      = kgrid,
        kshift     = (.5,.5,.5)
        )


# describe DFT run
# generate converged scf
    if first:
        scf = generate_pwscf(
            identifier     = 'scf',
            path           = directory+'/scf',
            job            = dftjob,
            input_type     = 'scf',
            system         = diamond,
            spin_polarized = False,
            pseudos        = ['Si.BFD_p.upf'],
            input_dft      = 'pbe',
            ecut           = 300,
            conv_thr       = 1e-8,
            mixing_beta    = .7,
            nosym          = False,
            wf_collect     = False,
            kgrid          = (12,12,12),
            kshift         = (0,0,0)
            )
        sims.append(scf)

# nscf run to generate orbitals
    nscf = generate_pwscf(
        identifier     = 'nscf',
        path           = directory+'/nscf_8_'+ks,
        job            = dftjob,
        input_type     = 'nscf',
        system         = diamond,
        spin_polarized = False,
        pseudos        = ['Si.BFD_p.upf'],
        input_dft      = 'pbe',
        ecut           = 300,
        conv_thr       = 1e-8,
        mixing_beta    = .7,
        tprnfor        = False,
        tstress        = False,
        nosym          = True,
        wf_collect     = True,
        dependencies   = (scf,'charge_density')
        )
    sims.append(nscf)

# describe orbital conversion run
    p2q = generate_pw2qmcpack(
        identifier   = 'p2q',
        path         = directory+'/nscf_8_'+ks,
        job          = p2qjob,
        write_psir   = False,
        dependencies = (nscf,'orbitals'),
        )
    sims.append(p2q)
# describe optimization run
# optimization inputs
    if first:
        linopt1 = linear(
            energy               = 0.0,
            unreweightedvariance = 1.0,
            reweightedvariance   = 0.0,
            timestep             = 0.4,
            samples              = 8192,
            warmupsteps          = 50,
            blocks               = 64,
            substeps             = 4,
            nonlocalpp           = True,
            usebuffer            = True,
            walkers              = 1,
            minwalkers           = 0.5,
            maxweight            = 1e9, 
            usedrift             = True,
            minmethod            = 'quartic',
            beta                 = 0.0,
            exp0                 = -16,
            bigchange            = 15.0,
            alloweddifference    = 1e-4,
            stepsize             = 0.2,
            stabilizerscale      = 1.0,
            nstabilizers         = 3
            )
        
        linopt2 = linopt1.copy()
        linopt2.samples = 16384
        
        # optimization run
        opt = generate_qmcpack(
            identifier   = 'opt',
            path         = directory+'/opt',
            job          = qmcjob,
            input_type   = 'basic',
            system       = diamond,
            twistnum     = 0,
            bconds       = 'ppp',
            pseudos      = ['Si.BFD_p.xml'],
            jastrows     = [('J1','bspline',8),
                            ('J2','bspline',8)],
            calculations = [loop(max=12,qmc=linopt1),
                            loop(max=12,qmc=linopt2)],
            dependencies = (p2q,'orbitals')
            )
        sims.append(opt)

    ntwists = kgrid[0]*kgrid[1]*kgrid[2]
# DMC run GPU
    qmcgpu = generate_qmcpack(
        identifier   = 'dmcgpu',
        path         = directory+'/dmcgpu_64_'+ks,
        job          = job(machine='daint',gpu=True,nodes=ntwists,hours=24,threads=12,processes_per_node=1,queue="normal",app=appdir+'qmcpack.gpu.double'),
        input_type   = 'basic',
        system       = diamond,
        bconds       = 'ppp',
        pseudos      = ['Si.BFD_p.xml'],
        jastrows     = [],            
        calculations = [
            vmc(
                walkers     =   128,
                gpu         = "yes",
                warmupsteps =  30,
                blocks      =  20,
                steps       =  10,
                substeps    =   2,
                timestep    =  .4,
                samples     = 2048
                ),
            dmc(
                checkpoint    = 5 ,
                walkers       =   128,
                gpu           = "yes",
                warmupsteps   =  20, 
                blocks        =  100,
                steps         =  20,
                timestep      = 0.04,
                nonlocalmoves = True
                )
            ],
        dependencies = [(p2q,'orbitals'),(opt,'jastrow')]
        )
    sims.append(qmcgpu)
# VMC run GPU
    qqmcgpu = generate_qmcpack(
        identifier   = 'vmcgpu',
        path         = directory+'/vmcgpu_64_'+ks,
        job          = job(machine='daint',gpu=True,nodes=ntwists,hours=24,threads=12,processes_per_node=1,queue="normal",app=appdir+'qmcpack.gpu.double'),
        input_type   = 'basic',
        system       = diamond,
        bconds       = 'ppp',
        pseudos      = ['Si.BFD_p.xml'],
        jastrows     = [],            
        calculations = [
            vmc(
                walkers     =   128,
                gpu         = "yes",
                warmupsteps =  30,
                blocks      =  1000,
                steps       =  10,
                substeps    =   2,
                timestep    =  .4,
                samples     = 2048
                )
            ],
        dependencies = [(p2q,'orbitals'),(opt,'jastrow')]
        )
    sims.append(qqmcgpu)
# DMC run CPU
    qmc = generate_qmcpack(
        identifier   = 'dmccpu',
        path         = directory+'/dmccpu_64_'+ks,
        job          = job(sub_options='-C gpu',nodes=ntwists,hours=24,threads=12,queue="normal",app=appdir+'qmcpack.cpu'),
        input_type   = 'basic',
        system       = diamond,
        bconds       = 'ppp',
        pseudos      = ['Si.BFD_p.xml'],
        jastrows     = [],            
        calculations = [
            vmc(
                walkers     =   1,
                warmupsteps =  30,
                blocks      =  20,
                steps       =  10,
                substeps    =   2,
                timestep    =  .4,
                samples     = 2048
                ),
            dmc(
                checkpoint    = 5 ,
                warmupsteps   =  20, 
                blocks        =  100,
                steps         =  20,
                timestep      = 0.04,
                nonlocalmoves = True
                )
            ],
        dependencies = [(p2q,'orbitals'),(opt,'jastrow')]
        )
    sims.append(qmc)
# VMC run CPU
    qqmccpu = generate_qmcpack(
        identifier   = 'vmccpu',
        path         = directory+'/vmccpu_64_'+ks,
        job          = job(sub_options='-C gpu',nodes=ntwists,hours=8,threads=12,queue="normal",app=appdir+'qmcpack.cpu'),
        input_type   = 'basic',
        system       = diamond,
        bconds       = 'ppp',
        pseudos      = ['Si.BFD_p.xml'],
        jastrows     = [],            
        calculations = [
            vmc(
                walkers     =  128,
                warmupsteps =  30,
                blocks      =  1000,
                steps       =  10,
                substeps    =   2,
                timestep    =  .4,
                samples     = 2048
                )
            ],
        dependencies = [(p2q,'orbitals'),(opt,'jastrow')]
        )
    sims.append(qqmccpu)
    first = False


# execute all simulations
run_project(sims) 
